/* $Id$ */

/* dict - a generic dictionnary, variation over the B-tree concept          */

/* Copyright (c) 2003, 2004 Martin Quinson. All rights reserved.            */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include "dict_private.h"  /* prototypes of this module */

XBT_LOG_EXTERNAL_CATEGORY(xbt_dict);
XBT_LOG_NEW_DEFAULT_SUBCATEGORY(xbt_dict_elm,xbt_dict,"Dictionaries internals");

XBT_LOG_NEW_SUBCATEGORY(xbt_dict_add,xbt_dict,"Dictionaries internals: elements addition");
XBT_LOG_NEW_SUBCATEGORY(xbt_dict_search,xbt_dict,"Dictionaries internals: searching");
XBT_LOG_NEW_SUBCATEGORY(xbt_dict_remove,xbt_dict,"Dictionaries internals: elements removal");
XBT_LOG_NEW_SUBCATEGORY(xbt_dict_collapse,xbt_dict,"Dictionaries internals: post-removal cleanup");

xbt_dictelm_t xbt_dictelm_new(const char *key,
			      int key_len,
			      void *content,
			      void_f_pvoid_t free_f,
			      xbt_dictelm_t next) {
  xbt_dictelm_t element = xbt_new0(s_xbt_dictelm_t, 1);
  
  element->key = xbt_new0(char, key_len + 1);
  strncpy(element->key, key, key_len);

  element->key_len = key_len;
  element->content = content;
  element->free_f = free_f;
  element->next = next;
  
  return element;
}

void xbt_dictelm_free(xbt_dictelm_t element) {
  if (element != NULL) {
    xbt_free(element->key);
    
    if (element->free_f != NULL && element->content != NULL) {
      element->free_f(element->content);
    }
   
    xbt_free(element);
  }
}
